/*
 * kgraft_patch_bsc1077404
 *
 * Fix for CVE-2017-18075, bsc#1077404
 *
 *  Upstream commit:
 *  d76c68109f37 ("crypto: pcrypt - fix freeing pcrypt instances")
 *
 *  SLE12(-SP1) commit:
 *  not affected
 *
 *  SLE12-SP2 commit:
 *  4.4.111 stable bff0355653759585ad51dad59d83040ccaafbd68
 *
 *  SLE12-SP3 commit:
 *  4.4.111 stable 199206ff9e3f4d3311ade2c79bc60b0fb8d624c1
 *
 *  Copyright (c) 2018 SUSE
 *  Author: Nicolai Stange <nstange@suse.de>
 *
 *  Based on the original Linux kernel code. Other copyrights apply.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include <linux/kernel.h>
#include <crypto/algapi.h>
#include <crypto/internal/aead.h>
#include "kgr_patch_bsc1077404.h"

/* from crypto/pcrypt.c */
struct pcrypt_instance_ctx {
	struct crypto_aead_spawn spawn;
	atomic_t tfm_count;
};


/* Patched */
void kgr_pcrypt_free(struct crypto_instance *inst)
{
	struct pcrypt_instance_ctx *ctx = crypto_instance_ctx(inst);

	crypto_drop_aead(&ctx->spawn);
	/*
	 * Fix CVE-2017-18075
	 *  -1 line, +1 line
	 */
	kfree(container_of(inst, struct aead_instance, s.base));
}
